@@ -1,21 +1,8 @@
 /*
-    Copyright (C) 2013 Mark Gaiser <markg85@gmail.com>
-    Copyright (C) 2016 Martin Klapetek <mklapetek@kde.org>
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+    SPDX-FileCopyrightText: 2013 Mark Gaiser <markg85@gmail.com>
+    SPDX-FileCopyrightText: 2016 Martin Klapetek <mklapetek@kde.org>
 
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include <QDebug>
@@ -41,7 +28,11 @@ Calendar::Calendar(QObject *parent)
 
 QDateTime Calendar::displayedDate() const
 {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+    return m_displayedDate.startOfDay();
+#else
     return QDateTime(m_displayedDate);
+#endif
 }
 
 void Calendar::setDisplayedDate(const QDate &dateTime)
@@ -74,7 +65,11 @@ void Calendar::setDisplayedDate(const QDateTime &dateTime)
 
 QDateTime Calendar::today() const
 {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+    return m_today.startOfDay();
+#else
     return QDateTime(m_today);
+#endif
 }
 
 void Calendar::setToday(const QDateTime &dateTime)
