@@ -1,22 +1,9 @@
 /*
- *   Copyright 2010 Marco Martin <mart@kde.org>
- *   Copyright 2014 David Edmundson <davidedmundson@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2010 Marco Martin <mart@kde.org>
+    SPDX-FileCopyrightText: 2014 David Edmundson <davidedmundson@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "framesvgitem.h"
 
@@ -145,11 +132,13 @@ public:
 
             //if tiling horizontally
             if (m_border == FrameSvg::TopBorder || m_border == FrameSvg::BottomBorder || m_border == FrameSvg::NoBorder) {
-                textureRect.setWidth(nodeRect.width() / m_elementNativeSize.width());
+                // cmp. CSS3's border-image-repeat: "repeat", though with first tile not centered, but aligned to left
+                textureRect.setWidth((qreal) nodeRect.width() / m_elementNativeSize.width());
             }
             //if tiling vertically
             if (m_border == FrameSvg::LeftBorder || m_border == FrameSvg::RightBorder || m_border == FrameSvg::NoBorder) {
-                textureRect.setHeight(nodeRect.height() / m_elementNativeSize.height());
+                // cmp. CSS3's border-image-repeat: "repeat", though with first tile not centered, but aligned to top
+                textureRect.setHeight((qreal) nodeRect.height() / m_elementNativeSize.height());
             }
         } else if (m_fitMode == Stretch) {
             QString prefix = m_frameSvg->frameSvg()->actualPrefix();
@@ -279,18 +268,23 @@ FrameSvgItem::~FrameSvgItem()
 class CheckMarginsChange
 {
 public:
-    CheckMarginsChange(FrameSvgItemMargins *margins)
-        : m_oldMargins(margins ? margins->margins() : QVector<qreal>()), m_margins(margins)
+    CheckMarginsChange(QVector<qreal>& oldMargins, FrameSvgItemMargins *marginsObject)
+        : m_oldMargins(oldMargins), m_marginsObject(marginsObject)
     {}
 
-    ~CheckMarginsChange() {
-        if (m_margins && m_margins->margins() != m_oldMargins) {
-            m_margins->update();
+    ~CheckMarginsChange()
+    {
+        const QVector<qreal> oldMarginsBefore = m_oldMargins;
+        m_oldMargins = m_marginsObject ? m_marginsObject->margins() : QVector<qreal>();
+
+        if (oldMarginsBefore != m_oldMargins) {
+            m_marginsObject->update();
         }
     }
 
-    const QVector<qreal> m_oldMargins;
-    FrameSvgItemMargins *const m_margins;
+private:
+    QVector<qreal>& m_oldMargins;
+    FrameSvgItemMargins *const m_marginsObject;
 };
 
 void FrameSvgItem::setImagePath(const QString &path)
@@ -299,8 +293,8 @@ void FrameSvgItem::setImagePath(const QString &path)
         return;
     }
 
-    CheckMarginsChange checkMargins(m_margins);
-    CheckMarginsChange checkFixedMargins(m_fixedMargins);
+    CheckMarginsChange checkMargins(m_oldMargins, m_margins);
+    CheckMarginsChange checkFixedMargins(m_oldFixedMargins, m_fixedMargins);
 
     updateDevicePixelRatio();
     m_frameSvg->setImagePath(path);
@@ -343,8 +337,8 @@ void FrameSvgItem::setPrefix(const QVariant &prefixes)
         return;
     }
 
-    CheckMarginsChange checkMargins(m_margins);
-    CheckMarginsChange checkFixedMargins(m_fixedMargins);
+    CheckMarginsChange checkMargins(m_oldMargins, m_margins);
+    CheckMarginsChange checkFixedMargins(m_oldFixedMargins, m_fixedMargins);
 
     m_prefixes = prefixList;
     applyPrefixes();
@@ -430,7 +424,7 @@ void FrameSvgItem::setEnabledBorders(const Plasma::FrameSvg::EnabledBorders bord
         return;
     }
 
-    CheckMarginsChange checkMargins(m_margins);
+    CheckMarginsChange checkMargins(m_oldMargins, m_margins);
 
     m_frameSvg->setEnabledBorders(borders);
     emit enabledBordersChanged();
@@ -448,14 +442,28 @@ bool FrameSvgItem::hasElementPrefix(const QString &prefix) const
     return m_frameSvg->hasElementPrefix(prefix);
 }
 
+
+QRegion FrameSvgItem::mask() const
+{
+    return m_frameSvg->mask();
+}
+
 void FrameSvgItem::geometryChanged(const QRectF &newGeometry,
                                    const QRectF &oldGeometry)
 {
-    if (isComponentComplete()) {
+    const bool isComponentComplete = this->isComponentComplete();
+    if (isComponentComplete) {
         m_frameSvg->resizeFrame(newGeometry.size());
         m_sizeChanged = true;
     }
+
     QQuickItem::geometryChanged(newGeometry, oldGeometry);
+
+    // the above only triggers updatePaintNode, so we have to inform subscribers
+    // about the potential change of the mask explicitly here
+    if (isComponentComplete) {
+        emit maskChanged();
+    }
 }
 
 void FrameSvgItem::doUpdate()
@@ -464,8 +472,8 @@ void FrameSvgItem::doUpdate()
         return;
     }
 
-    CheckMarginsChange checkMargins(m_margins);
-    CheckMarginsChange checkFixedMargins(m_fixedMargins);
+    CheckMarginsChange checkMargins(m_oldMargins, m_margins);
+    CheckMarginsChange checkFixedMargins(m_oldFixedMargins, m_fixedMargins);
 
     //if the theme changed, the available prefix may have changed as well
     applyPrefixes();
@@ -493,6 +501,7 @@ void FrameSvgItem::doUpdate()
 
     update();
 
+    emit maskChanged();
     emit repaintNeeded();
 }
 
@@ -509,6 +518,8 @@ QSGNode *FrameSvgItem::updatePaintNode(QSGNode *oldNode, QQuickItem::UpdatePaint
         return nullptr;
     }
 
+    const QSGTexture::Filtering filtering = smooth() ? QSGTexture::Linear : QSGTexture::Nearest;
+
     if (m_fastPath) {
         if (m_textureChanged) {
             delete oldNode;
@@ -556,6 +567,12 @@ QSGNode *FrameSvgItem::updatePaintNode(QSGNode *oldNode, QQuickItem::UpdatePaint
             m_textureChanged = false;
         }
 
+        QSGNode *node = oldNode->firstChild();
+        while (node) {
+            static_cast<FrameItemNode *>(node)->setFiltering(filtering);
+            node = node->nextSibling();
+        }
+
         if (m_sizeChanged) {
             FrameNode* frameNode = static_cast<FrameNode*>(oldNode);
             QSize frameSize(width(), height());
@@ -573,10 +590,10 @@ QSGNode *FrameSvgItem::updatePaintNode(QSGNode *oldNode, QQuickItem::UpdatePaint
         if (!textureNode) {
             delete oldNode;
             textureNode = new ManagedTextureNode;
-            textureNode->setFiltering(QSGTexture::Nearest);
             m_textureChanged = true; //force updating the texture on our newly created node
             oldNode = textureNode;
         }
+        textureNode->setFiltering(filtering);
 
         if ((m_textureChanged || m_sizeChanged) || textureNode->texture()->textureSize() != m_frameSvg->size()) {
             QImage image = m_frameSvg->framePixmap().toImage();
@@ -599,8 +616,8 @@ void FrameSvgItem::classBegin()
 
 void FrameSvgItem::componentComplete()
 {
-    CheckMarginsChange checkMargins(m_margins);
-    CheckMarginsChange checkFixedMargins(m_fixedMargins);
+    CheckMarginsChange checkMargins(m_oldMargins, m_margins);
+    CheckMarginsChange checkFixedMargins(m_oldFixedMargins, m_fixedMargins);
 
     QQuickItem::componentComplete();
     m_frameSvg->resizeFrame(QSize(width(), height()));
