@@ -1,23 +1,10 @@
 /*
- *   Copyright 2006-2007 by Aaron Seigo <aseigo@kde.org>
- *   Copyright 2007 by Riccardo Iaconelli <riccardo@kde.org>
- *   Copyright 2008 by Ménard Alexis <darktears31@gmail.com>
+    SPDX-FileCopyrightText: 2006-2007 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2007 Riccardo Iaconelli <riccardo@kde.org>
+    SPDX-FileCopyrightText: 2008 Ménard Alexis <darktears31@gmail.com>
 
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #ifndef PLASMA_APPLET_H
 #define PLASMA_APPLET_H
@@ -26,8 +13,8 @@
 #include <QKeySequence>
 #include <QUrl>
 
-#include <kconfiggroup.h>
-#include <kplugininfo.h>
+#include <KConfigGroup>
+#include <KPluginInfo>
 
 #include <plasma/plasma.h>
 #include <plasma/version.h>
@@ -75,6 +62,21 @@ class PLASMA_EXPORT Applet : public QObject
     Q_PROPERTY(QString icon READ icon WRITE setIcon NOTIFY iconChanged FINAL)
     Q_PROPERTY(bool busy READ isBusy WRITE setBusy NOTIFY busyChanged FINAL)
 
+    /**
+     * How the applet wants its background to be drawn. The containment may chose to ignore this hint.
+     */
+    Q_PROPERTY(Plasma::Types::BackgroundHints backgroundHints WRITE setBackgroundHints READ backgroundHints NOTIFY backgroundHintsChanged FINAL)
+
+    /**
+     * The containment (and/or the user) may decide to use another kind of background instead (if supported by the applet)
+     */
+    Q_PROPERTY(Plasma::Types::BackgroundHints userBackgroundHints WRITE setUserBackgroundHints READ userBackgroundHints NOTIFY userBackgroundHintsChanged FINAL)
+
+    /**
+     * The effective background hints the applet has, internally decided how to mix with userBackgroundHints
+     */
+    Q_PROPERTY(Plasma::Types::BackgroundHints effectiveBackgroundHints READ effectiveBackgroundHints NOTIFY effectiveBackgroundHintsChanged FINAL)
+
 public:
 //CONSTRUCTORS
     /**
@@ -86,6 +88,7 @@ public:
      */
     explicit Applet(QObject *parent = nullptr, const QString &serviceId = QString(), uint appletId = 0);
 
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 28)
     /**
      * @param parent the QObject this applet is parented to
      * @param info the plugin information object for this Applet
@@ -93,9 +96,11 @@ public:
      *      instances of the same Applet type
      * @since 4.6
      *
-     * @deprecated prefer using KPluginMetaData
+     * @deprecated Since 5.28, prefer using KPluginMetaData
      */
-    PLASMA_DEPRECATED explicit Applet(const KPluginInfo &info, QObject *parent = nullptr, uint appletId = 0);
+    PLASMA_DEPRECATED_VERSION(5, 28, "Use Applet(const KPluginMetaData &, QObject *, uint")
+    explicit Applet(const KPluginInfo &info, QObject *parent = nullptr, uint appletId = 0);
+#endif
 
     /**
      * @param parent the QObject this applet is parented to
@@ -106,7 +111,7 @@ public:
      */
     explicit Applet(const KPluginMetaData &metadata, QObject *parent = nullptr, uint appletId = 0);
 
-    ~Applet();
+    ~Applet() override;
 
 //BOOKKEEPING
     /**
@@ -248,15 +253,16 @@ public:
     void setUserConfiguring(bool configuring);
 
 //UTILS
-#ifndef PLASMA_NO_DEPRECATED
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 6)
     /**
      * Accessor for the associated Package object if any.
      * Generally, only Plasmoids come in a Package.
      *
-     * @deprecated use kPackage() instead
      * @return the Package object, or an invalid one if none
+     * @deprecated Since 5.6, use kPackage() instead
      **/
-    PLASMA_DEPRECATED Package package() const;
+    PLASMA_DEPRECATED_VERSION(5, 6, "Use Applet::kPackage()")
+    Package package() const;
 #endif
 
     /**
@@ -278,13 +284,16 @@ public:
     void updateConstraints(Plasma::Types::Constraints constraints = Plasma::Types::AllConstraints);
 
 //METADATA
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 28)
     /**
      * @return metadata information about this plugin
      * @see KPluginInfo, pluginMetaData
      * @since 5.0
-     * @deprecated use pluginMetaData instead
+     * @deprecated Since 5.28, use pluginMetaData instead
      */
-    PLASMA_DEPRECATED KPluginInfo pluginInfo() const;
+    PLASMA_DEPRECATED_VERSION(5, 28, "Use Applet::pluginMetaData()")
+    KPluginInfo pluginInfo() const;
+#endif
 
     /**
      * @return metadata information about this plugin
@@ -311,6 +320,7 @@ public:
      */
     void setTitle(const QString &title);
 
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 19)
     /**
      * Attempts to load an applet from a package
      *
@@ -326,10 +336,12 @@ public:
      * @return a pointer to the loaded applet, or 0 on load failure
      * @since 4.3
      *
-     * @deprecated use Containment::createApplet() instead, you are not
+     * @deprecated Since 5.19, use Containment::createApplet() instead, you are not
      * supposed to have applets without containments
      **/
-    PLASMA_DEPRECATED static Applet *loadPlasmoid(const QString &path, uint appletId = 0);
+    PLASMA_DEPRECATED_VERSION(5, 19, "Use Containment::createApplet(...)")
+    static Applet *loadPlasmoid(const QString &path, uint appletId = 0);
+#endif
 
     /**
      * @returns The icon name related to this applet
@@ -360,6 +372,38 @@ public:
      */
     void setBusy(bool busy);
 
+    /**
+     * How the applet wants its background to be drawn. The containment may chose to ignore this hint.
+     * @since 5.65
+     */
+    Plasma::Types::BackgroundHints backgroundHints() const;
+
+    /**
+     * Sets the applet background hints. Only Applet implementations should write this property
+     * @since 5.65
+     */
+    void setBackgroundHints(Plasma::Types::BackgroundHints hint);
+
+    /**
+     * The containment (and/or the user) may decide to use another kind of background instead if supported by the applet.
+     * In order for an applet to support user configuration of the
+     * background, it needs to have the Plasma::Types::ConfigurableBackground flag set in its backgroundHints
+     * @since 5.65
+     */
+    Plasma::Types::BackgroundHints userBackgroundHints() const;
+
+    /**
+     * Sets the hints the user wished the background style for the applet to be.
+     * @since 5.65
+     */
+    void setUserBackgroundHints(Plasma::Types::BackgroundHints hint);
+
+    /**
+     * The effective background hints the applet will have: it will follow userBackgroundHints only if backgroundHints has the Plasma::Types::ConfigurableBackground flag set
+     * @since 5.65
+     */
+    Plasma::Types::BackgroundHints effectiveBackgroundHints() const;
+
 //ACTIONS
     /**
      * Returns a list of context-related QAction instances.
@@ -383,7 +427,7 @@ public:
     void setGlobalShortcut(const QKeySequence &shortcut);
 
     /**
-     * @return the global shortcut associated with this wiget, or
+     * @return the global shortcut associated with this widget, or
      * an empty shortcut if no global shortcut is associated.
      */
     QKeySequence globalShortcut() const;
@@ -474,6 +518,24 @@ Q_SIGNALS:
      */
     void busyChanged(bool busy);
 
+    /**
+     * Emitted when the background hints have changed
+     * @since 5.65
+     */
+    void backgroundHintsChanged();
+
+    /**
+     * Emitted when the user background hints have changed
+     * @since 5.65
+     */
+    void userBackgroundHintsChanged();
+
+    /**
+     * Emitted when the effective background hints have changed
+     * @since 5.65
+     */
+    void effectiveBackgroundHintsChanged();
+
 //CONFIGURATION
     /**
      * Emitted when an applet has changed values in its configuration
@@ -500,7 +562,7 @@ Q_SIGNALS:
 
     /**
      * Emitted when activation is requested due to, for example, a global
-     * keyboard shortcut. By default the wiget is given focus.
+     * keyboard shortcut. By default the widget is given focus.
      */
     void activated();
 
@@ -568,6 +630,7 @@ public Q_SLOTS:
     /**
      * Called when applet configuration values have changed.
      */
+    //TODO KF6: make it not a slot anymore and protected
     virtual void configChanged();
 
 //UTILS
@@ -675,7 +738,7 @@ protected:
     /**
      * Reimplemented from QObject
      */
-    void timerEvent(QTimerEvent *event) Q_DECL_OVERRIDE;
+    void timerEvent(QTimerEvent *event) override;
 
 private:
     /**
