@@ -1,28 +1,15 @@
 /*
- *   Copyright 2007 by Aaron Seigo <aseigo@kde.org>
- *   Copyright 2008 by Ménard Alexis <darktears31@gmail.com>
- *   Copyright (c) 2009 Chani Armitage <chani@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2007 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2008 Ménard Alexis <darktears31@gmail.com>
+    SPDX-FileCopyrightText: 2009 Chani Armitage <chani@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #ifndef PLASMA_CONTAINMENT_H
 #define PLASMA_CONTAINMENT_H
 
-#include <ksharedconfig.h>
+#include <KSharedConfig>
 #include <plasma/applet.h>
 
 namespace Plasma
@@ -42,7 +29,7 @@ class ContainmentPrivate;
  * Containment objects provide the means to group applets into functional sets.
  * They also provide the following:
  *
- * creation of focussing event
+ * creation of focusing event
  * - drawing of the background image (which can be interactive)
  * - form factors (e.g. panel, desktop, full screen, etc)
  * - applet layout management
@@ -85,12 +72,12 @@ public:
      */
     Containment(QObject *parent, const QVariantList &args);
 
-    ~Containment();
+    ~Containment() override;
 
     /**
      * Reimplemented from Applet
      */
-    void init() Q_DECL_OVERRIDE;
+    void init() override;
 
     /**
      * Returns the type of containment
@@ -145,13 +132,13 @@ public:
      * @reimp
      * @sa Applet::save(KConfigGroup &)
      */
-    void save(KConfigGroup &group) const Q_DECL_OVERRIDE;
+    void save(KConfigGroup &group) const override;
 
     /**
      * @reimp
      * @sa Applet::restore(KConfigGroup &)
      */
-    void restore(KConfigGroup &group) Q_DECL_OVERRIDE;
+    void restore(KConfigGroup &group) override;
 
     /**
      * Sets wallpaper plugin.
